@@ -1,7 +1,7 @@
 /*
     ccid.h: CCID structures
     Copyright (C) 2003-2010   Ludovic Rousseau
-    Copyright (C) 2009-2023   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2025   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -304,6 +304,7 @@ typedef struct
 #define ACS_ACR1252IMP_1S_CL_READER		0x072f2259
 #define ACS_WALLETMATE_1S_CL_READER		0x072f226b
 #define ACS_WALLETMATE_II_1S_CL_READER	0x072f2307
+#define ACS_WALLETMATE_II_2S_CL_READER	0x072f2401
 #define ACS_ACR1261_1S_DUAL_READER		0x072f2211
 #define ACS_ACR128U						0x072f2100
 #define ACS_ACR1281_DUAL_READER_QPBOC	0x072f2208
@@ -321,6 +322,7 @@ typedef struct
 #define ACS_ACR1552_1S_CL_READER		0x072f2303
 #define ACS_ACR1555_1S_CL_READER		0x072f230a
 #define ACS_ACR1581_1S_DUAL_READER		0x072f2301
+#define ACS_AIR60_1S_DUAL_READER		0x072f230b
 #define ACS_AET62_PICC_READER			0x072f0102
 #define ACS_AET62_1SAM_PICC_READER		0x072f0103
 #define ACS_APG8201						0x072f8201
