@@ -1,7 +1,7 @@
 /*
 	ccid_usb.c: USB access routines using the libusb library
 	Copyright (C) 2003-2010	Ludovic Rousseau
-	Copyright (C) 2009-2023	Advanced Card Systems Ltd.
+	Copyright (C) 2009-2025	Advanced Card Systems Ltd.
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Lesser General Public
@@ -473,6 +473,10 @@ again_libusb:
 					|| (ACS_ACR1252IMP_1S_CL_READER == readerID)
 					|| (ACS_WALLETMATE_1S_CL_READER == readerID)
 					|| (ACS_ACR1581_1S_DUAL_READER == readerID)
+					|| (ACS_AIR60_1S_DUAL_READER == readerID)
+					|| (ACS_WALLETMATE_II_1S_CL_READER == readerID)
+					|| (ACS_WALLETMATE_II_2S_CL_READER == readerID)
+					|| (ACS_ACR1555_1S_CL_READER == readerID)
 					|| (ACS_ACR1552_1S_CL_READER == readerID))
 				{
 						/*
@@ -522,11 +526,15 @@ again_libusb:
 						|| (ACS_ACR1252_1S_CL_READER == readerID)
 						|| (ACS_ACR1252IMP_1S_CL_READER == readerID)
 						|| (ACS_WALLETMATE_1S_CL_READER == readerID)
+						|| (ACS_WALLETMATE_II_1S_CL_READER == readerID)
+						|| (ACS_ACR1555_1S_CL_READER == readerID)
 						|| (ACS_ACR1552_1S_CL_READER == readerID))
 						/* only 2 interfaces for this device */
 						max_interface_number = 1;
 
-					if (ACS_ACR1581_1S_DUAL_READER == readerID)
+					if ((ACS_ACR1581_1S_DUAL_READER == readerID)
+						|| (ACS_AIR60_1S_DUAL_READER == readerID)
+						|| (ACS_WALLETMATE_II_2S_CL_READER == readerID))
 						/* 3 interfaces for this device */
 						max_interface_number = 2;
 
@@ -794,6 +802,10 @@ again:
 					|| (ACS_ACR1252IMP_1S_CL_READER == readerID)
 					|| (ACS_WALLETMATE_1S_CL_READER == readerID)
 					|| (ACS_ACR1581_1S_DUAL_READER == readerID)
+					|| (ACS_AIR60_1S_DUAL_READER == readerID)
+					|| (ACS_WALLETMATE_II_1S_CL_READER == readerID)
+					|| (ACS_WALLETMATE_II_2S_CL_READER == readerID)
+					|| (ACS_ACR1555_1S_CL_READER == readerID)
 					|| (ACS_ACR1552_1S_CL_READER == readerID))
 				{
 					/* use the next interface for the next "slot" */
@@ -1062,6 +1074,9 @@ again:
 					(readerID == ACS_ACR1252_1S_CL_READER) ||
 					(readerID == ACS_ACR1252IMP_1S_CL_READER) ||
 					(readerID == ACS_WALLETMATE_1S_CL_READER) ||
+					(readerID == ACS_WALLETMATE_II_1S_CL_READER) ||
+					(readerID == ACS_WALLETMATE_II_2S_CL_READER) ||
+					(readerID == ACS_ACR1555_1S_CL_READER) ||
 					(readerID == ACS_ACR1552_1S_CL_READER))
 				{
 					if (interface == 1)
@@ -1069,7 +1084,9 @@ again:
 				}
 
 				// The 3rd interface (composite device) is a SAM slot
-				if (readerID == ACS_ACR1581_1S_DUAL_READER)
+				if ((readerID == ACS_ACR1581_1S_DUAL_READER)
+					|| (readerID == ACS_AIR60_1S_DUAL_READER)
+					|| (readerID == ACS_WALLETMATE_II_2S_CL_READER))
 				{
 					if (interface == 2)
 						usbDevice[reader_index].ccid.isSamSlot = TRUE;
