@@ -27,3 +27,9 @@ int GetNewReaderIndex(const int Lun);
 int LunToReaderIndex(int Lun);
 void ReleaseReaderIndex(const int idx);
 
+uint16_t get_U16(void *);
+uint32_t get_U32(void *);
+void set_U16(void *, uint16_t);
+void set_U32(void *, uint32_t);
+void p_bswap_16(void *ptr);
+void p_bswap_32(void *ptr);
